@@ -21,16 +21,23 @@
 
 #include <config.h>
 
-#include <glib.h>
-
+#include <errno.h>
+#include <locale.h>
 #include <stdlib.h>
 #include <time.h>
+#include <unistd.h>
+
+#include <glib.h>
+#include <glib/gstdio.h>
+
 #include <gdk/gdkx.h>
 
 #ifdef WITH_SMCLIENT
 #include "eggsmclient.h"
+#ifdef GDK_WINDOWING_X11
 #include "eggdesktopfile.h"
 #endif
+#endif
 
 #include <dbus/dbus-protocol.h>
 #include <dbus/dbus-glib.h>
@@ -43,9 +50,9 @@
 #include "terminal-options.h"
 #include "terminal-util.h"
 
-#define TERMINAL_FACTORY_SERVICE_NAME   "org.gnome.Terminal.Factory"
-#define TERMINAL_FACTORY_SERVICE_PATH   "/org/gnome/Terminal/Factory"
-#define TERMINAL_FACTORY_INTERFACE_NAME "org.gnome.Terminal.Factory"
+#define TERMINAL_FACTORY_SERVICE_NAME_PREFIX  "org.gnome.Terminal.Display"
+#define TERMINAL_FACTORY_SERVICE_PATH         "/org/gnome/Terminal/Factory"
+#define TERMINAL_FACTORY_INTERFACE_NAME       "org.gnome.Terminal.Factory"
 
 #define TERMINAL_TYPE_FACTORY             (terminal_factory_get_type ())
 #define TERMINAL_FACTORY(object)          (G_TYPE_CHECK_INSTANCE_CAST ((object), TERMINAL_TYPE_FACTORY, TerminalFactory))
@@ -206,6 +213,29 @@ about_email_hook (GtkAboutDialog *about,
   g_free (uri);
 }
 
+static char *
+get_factory_name_for_display (const char *display_name)
+{
+  GString *name;
+  const char *p;
+
+  name = g_string_sized_new (strlen (TERMINAL_FACTORY_SERVICE_NAME_PREFIX) + strlen (display_name) + 1 /* NUL */);
+  g_string_append (name, TERMINAL_FACTORY_SERVICE_NAME_PREFIX);
+
+  for (p = display_name; *p; ++p)
+    {
+      if (g_ascii_isalnum (*p))
+        g_string_append_c (name, *p);
+      else
+        g_string_append_c (name, '_');
+    }
+
+  _terminal_debug_print (TERMINAL_DEBUG_FACTORY,
+                         "Factory name is \"%s\"\n", name->str);
+
+  return g_string_free (name, FALSE);
+}
+
 /* Evil hack alert: this is exported from libgconf-2 but not in a public header */
 extern gboolean gconf_ping_daemon (void);
          
@@ -215,19 +245,24 @@ main (int argc, char **argv)
   int i;
   char **argv_copy;
   int argc_copy;
-  const char *startup_id;
-  const char *display_name;
+  const char *startup_id, *display_name;
   GdkDisplay *display;
   TerminalOptions *options;
   DBusGConnection *connection;
+  char *factory_name = NULL;
   DBusGProxy *proxy;
   guint32 request_name_ret;
   GError *error = NULL;
 
+  setlocale (LC_ALL, "");
+
   bindtextdomain (GETTEXT_PACKAGE, TERM_LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
+  /* GConf uses ORBit2 which need GThread. See bug #565516 */
+  g_thread_init (NULL);
+
   _terminal_debug_init ();
 
   /* Make a NULL-terminated copy since we may need it later */
@@ -310,8 +345,9 @@ main (int argc, char **argv)
                                G_CALLBACK (name_owner_changed), factory, NULL);
 #endif
 
+  factory_name = get_factory_name_for_display (display_name);
   if (!org_freedesktop_DBus_request_name (proxy,
-                                          TERMINAL_FACTORY_SERVICE_NAME,
+                                          factory_name,
                                           DBUS_NAME_FLAG_DO_NOT_QUEUE,
                                           &request_name_ret,
                                           &error))
@@ -334,6 +370,9 @@ main (int argc, char **argv)
       gboolean retval;
       int ret = EXIT_SUCCESS;
 
+      _terminal_debug_print (TERMINAL_DEBUG_FACTORY,
+                             "Forwarding arguments to existing instance\n");
+
       env = g_listenv ();
       envc = g_strv_length (env);
       env_ptr_array = g_ptr_array_sized_new (envc);
@@ -356,7 +395,7 @@ main (int argc, char **argv)
       argv_array = terminal_util_strv_to_array (argc_copy, argv_copy);
 
       proxy = dbus_g_proxy_new_for_name (connection,
-                                         TERMINAL_FACTORY_SERVICE_NAME,
+                                         factory_name,
                                          TERMINAL_FACTORY_SERVICE_PATH,
                                          TERMINAL_FACTORY_INTERFACE_NAME);
       retval = org_gnome_Terminal_Factory_handle_arguments (proxy,
@@ -405,6 +444,7 @@ main (int argc, char **argv)
 
 factory_disabled:
   g_free (argv_copy);
+  g_free (factory_name);
 
   /* If the gconf daemon isn't available (e.g. because there's no dbus
    * session bus running), we'd crash later on. Tell the user about it
@@ -421,7 +461,7 @@ factory_disabled:
   gtk_about_dialog_set_url_hook (about_url_hook, NULL, NULL);
   gtk_about_dialog_set_email_hook (about_email_hook, NULL, NULL);
 
-#ifdef WITH_SMCLIENT
+#if defined(WITH_SMCLIENT) && defined(GDK_WINDOWING_X11)
   {
     char *desktop_file;
 
@@ -437,9 +477,15 @@ factory_disabled:
   terminal_app_initialize (options->use_factory);
   g_signal_connect (terminal_app_get (), "quit", G_CALLBACK (gtk_main_quit), NULL);
 
-  terminal_app_handle_options (terminal_app_get (), options, NULL);
+  terminal_app_handle_options (terminal_app_get (), options, TRUE /* allow resume */, NULL);
   terminal_options_free (options);
 
+  /* Now change directory to / so we don't prevent unmounting, e.g. if the
+   * factory is started by nautilus-open-terminal. See bug #565328.
+   */
+  if (chdir ("/") < 0)
+    g_warning ("Failed to chdir to /: %s", g_strerror (errno));
+
   gtk_main ();
 
   terminal_app_shutdown ();
@@ -455,7 +501,7 @@ factory_disabled:
 static gboolean
 handle_new_terminal_event (TerminalOptions *options)
 {
-  terminal_app_handle_options (terminal_app_get (), options, NULL);
+  terminal_app_handle_options (terminal_app_get (), options, FALSE /* no resume */, NULL);
 
   return FALSE;
 }
